#ifndef __DCL_DATE_TIME_H__
#define __DCL_DATE_TIME_H__		20050323

#ifndef __DCL_CONFIG_H__
#include <dcl/Config.h>
#endif
#ifndef __DCL_STRING_H__
#include <dcl/String.h>
#endif
#ifndef __DCL_EXCEPTION_H__
#include <dcl/Exception.h>
#endif

__DCL_BEGIN_NAMESPACE

class Date;
class Time;
class Interval;
class DateTime;

#define DATETIME_FORMAT_BUFFER_SIZE		256
// valid range -9999-01-01 ~ 9999-12-31
// internal range -599999-01-01 ~ 600000-12-31
class DCLCAPI Date
{
public:
	Date();
	Date(const Date& src);
	Date(int nYear, int nMonth, int nDay);

	void assign(int nYear, int nMonth, int nDay);

	// nDays == 0 : NULL
	// nDays > 0 : 0001-01-01 ~
	// nDays < 0 : ~ -0001-12-31
	Date(long nDays);
	void assign(long nDays);
	long days() const;

//	long jdays() const { return m_nJDay; }

	const Date& operator ++ ();
	const Date operator ++ (int);
	const Date& operator -- ();
	const Date operator -- (int);

	const Date& operator = (const Date& src);
	const Date& operator += (long nDays);
	const Date& operator -= (long nDays);
	
	void decode(int& nYear, int& nMonth, int& nDay) const;

	int year() const;
	int month() const;
	int day() const;

	int dayOfWeek() const;
	int dayOfYear() const;
	int daysInMonth() const;
	int daysInYear() const;

	// yyyy-mm-dd
	String toString() const;

	// strftime
	static const wchar_t* FORMAT_STRING;	// ISO 8601 format %Y-%m-%d

	String toStringF(const wchar_t* pszFormat = NULL) const;


	// 0001-01-01 <= date <= 9999-12-31
	static bool isValid(int y, int m, int d);

	static bool isLeapYear(int nYear);

protected:
	long	m_nJDay;

	static long convertGregorianToJulian(
				int nYear, int nMonth, int nDay
				);

	static void convertJulianToGregorian(
				long uJulianDays,
				int& nYear, int& nMonth, int& nDay
				);
};

DCLCAPI Date operator + (const Date& d, long nDays);
DCLCAPI Date operator + (long nDays, const Date& d);	
DCLCAPI Date operator - (const Date& d, long nDays);

// return days
DCLCAPI long operator - (const Date& d1, const Date& d2);

DCLCAPI bool operator == (const Date& d1, const Date& d2);
DCLCAPI bool operator != (const Date& d1, const Date& d2);
DCLCAPI bool operator >= (const Date& d1, const Date& d2);
DCLCAPI bool operator <= (const Date& d1, const Date& d2);
DCLCAPI bool operator > (const Date& d1, const Date& d2);
DCLCAPI bool operator < (const Date& d1, const Date& d2);

class DCLCAPI Time
{
public:
	Time();
	Time(const Time& src);
	Time(int nHour, int nMin, int nSec, int nMSec = 0);
	Time(long nTotalMSecs);

	void assign(int nHour, int nMin, int nSec, int nMSec = 0);
	void assign(long nTotalMSecs);

	const Time& operator = (const Time& src);
	const Time& operator += (long nMilliSeconds);
	const Time& operator -= (long nMilliSeconds);

	void decode(int& nHour, int& nMin, int& nSec, int& nMSec) const;

	int hour() const;		// 0 ~ 23
	int minute() const;		// 0 ~ 59
	int second() const;		// 0 ~ 59
	int msecond() const;	// 0 ~ 999

	// HH:MM:SS.MS 00:00:00.000
	String toString() const;

	// strftime
	static const wchar_t* FORMAT_STRING;

	String toStringF(const wchar_t* pszFormat = NULL);

	long totalMilliSeconds() const;	// 0 ~ (86400000 - 1)

	static bool isValid(int nHour, int nMin, int nSec, int nMSec);


protected:
	unsigned long	m_uMilliSeconds;
};

DCLCAPI Time operator + (const Time& t, long nMilliSeconds);
DCLCAPI Time operator + (long nMilliSeconds, const Time& t);
DCLCAPI Time operator - (const Time& t, long nMilliSeconds);

// return milliseconds
DCLCAPI long operator - (const Time& t1, const Time& t2);

DCLCAPI bool operator == (const Time& t1, const Time& t2);
DCLCAPI bool operator != (const Time& t1, const Time& t2);
DCLCAPI bool operator >= (const Time& t1, const Time& t2);
DCLCAPI bool operator <= (const Time& t1, const Time& t2);
DCLCAPI bool operator > (const Time& t1, const Time& t2);
DCLCAPI bool operator < (const Time& t1, const Time& t2);

class DCLCAPI Interval
{
public:
	Interval();
	Interval(const Interval& src);
	Interval(int nDays, int nMilliSeconds);
	Interval(int nDays,
			int nHours, int nMinutes, int nSeconds,
			int nMilliSeconds = 0
			);
	Interval(int64_t nTotalMilliSeconds);

	void assign(int nDays, int nMilliSeconds);
	void assign(int nDays,
			int nHours, int nMinutes, int nSeconds,
			int nMilliSeconds = 0
			);
	void assign(int64_t nTotalMilliSeconds);

	const Interval& operator = (const Interval& src);
	const Interval& operator += (const Interval& iv);
	const Interval& operator -= (const Interval& iv);

	void decode(long& nDays,
			int& nHours, int& nMinutes, int& nSeconds,
			int& nMilliSeconds
			) const;
	void decode(long& nDays, long& nMilliSeconds);

	long days() const;
	int hour() const;		// -23 ~ 23
	int minute() const;		// -59 ~ 59
	int second() const;		// -59 ~ 59
	int msecond() const;	// -999 ~ 999

	int64_t totalHours() const;
	int64_t totalMinutes() const;
	int64_t totalSeconds() const;
	int64_t totalMilliSeconds() const;

	// [-]000 days [-]00:00:00.000
	String toString() const;

protected:
	int64_t	m_nMilliSeconds;
};

DCLCAPI Interval operator + (const Interval& iv1, const Interval& iv2);
DCLCAPI Interval operator - (const Interval& iv1, const Interval& iv2);

DCLCAPI bool operator == (const Interval& iv1, const Interval& iv2);
DCLCAPI bool operator != (const Interval& iv1, const Interval& iv2);
DCLCAPI bool operator >= (const Interval& iv1, const Interval& iv2);
DCLCAPI bool operator <= (const Interval& iv1, const Interval& iv2);
DCLCAPI bool operator > (const Interval& iv1, const Interval& iv2);
DCLCAPI bool operator < (const Interval& iv1, const Interval& iv2);

class DCLCAPI DateTime
{
public:
	DateTime() { }
	DateTime(const DateTime& src);
	DateTime(time_t timer);
	DateTime(const Date& date, const Time& time);
	DateTime(int nYear, int nMonth, int nDay,
			int nHour, int nMin, int nSec, int nMSec = 0);

	void assign(time_t time);
	void assign(const Date& date, const Time& time);
	void assign(int nYear, int nMonth, int nDay,
			int nHour, int nMin, int nSec, int nMSec = 0);

	const DateTime& operator = (const DateTime& src);
	const DateTime& operator += (const Interval& iv);
	const DateTime& operator -= (const Interval& iv);

	// Date::toString() + Time::toString()
	String toString() const;

	// stftime
	static const wchar_t* FORMAT_STRING;

	String toStringF(const wchar_t* pszFormat = NULL) const;

	Date& date();
	const Date& date() const;

	Time& time();
	const Time& time() const;

	static int compare(const DateTime& dt1, const DateTime& dt2);

	static int64_t getCurrentTimeMillis();
	static DateTime getCurrentUTCTime();
	static DateTime getCurrentLocalTime();
	
/*
	static time_t parse(const char* pszDateTimeString)
				__DCL_THROWS1(ParseException*);

	// strptime
	static DateTime parseF(
			const wchar_t* pszDateTimeString,
			const wchar_t* pszFormat
			) __DCL_THROWS1(ParseException*);
*/

protected:
	Date	m_date;
	Time	m_time;

};

DCLCAPI DateTime operator + (const DateTime& dt, const Interval& iv);
DCLCAPI DateTime operator + (const Interval& iv, const DateTime& dt);
DCLCAPI DateTime operator - (const DateTime& dt, const Interval& iv);
DCLCAPI Interval operator - (const DateTime& dt1, const DateTime& dt2);

DCLCAPI bool operator == (const DateTime& dt1, const DateTime& dt2);
DCLCAPI bool operator != (const DateTime& dt1, const DateTime& dt2);
DCLCAPI bool operator >= (const DateTime& dt1, const DateTime& dt2);
DCLCAPI bool operator <= (const DateTime& dt1, const DateTime& dt2);
DCLCAPI bool operator > (const DateTime& dt1, const DateTime& dt2);
DCLCAPI bool operator < (const DateTime& dt1, const DateTime& dt2);


#include "DateTime.inl"

__DCL_END_NAMESPACE

#endif	// __DCL_DATE_TIME_H__

